#!/bin/bash

# TransmissionRssManager Installer Script for Linux
# This script installs the TransmissionRssManager application and its dependencies

# Colors for output
GREEN='\033[0;32m'
YELLOW='\033[1;33m'
RED='\033[0;31m'
NC='\033[0m' # No Color

# Error handling
set -e
trap 'echo -e "${RED}An error occurred. Installation failed.${NC}"; exit 1' ERR

# Check if script is run as root
if [ "$EUID" -eq 0 ]; then
  echo -e "${YELLOW}Warning: It's recommended to run this script as a regular user with sudo privileges, not as root.${NC}"
  read -p "Continue anyway? (y/n) " -n 1 -r
  echo
  if [[ ! $REPLY =~ ^[Yy]$ ]]; then
    exit 1
  fi
fi

# Detect Linux distribution
if [ -f /etc/os-release ]; then
    . /etc/os-release
    DISTRO=$ID
else
    echo -e "${RED}Cannot detect Linux distribution. Exiting.${NC}"
    exit 1
fi

echo -e "${GREEN}Installing TransmissionRssManager on $PRETTY_NAME...${NC}"

# Install .NET SDK and runtime
install_dotnet() {
    echo -e "${GREEN}Installing .NET SDK...${NC}"
    
    case $DISTRO in
        ubuntu|debian|linuxmint)
            # Add Microsoft package repository
            wget -O packages-microsoft-prod.deb https://packages.microsoft.com/config/$DISTRO/$VERSION_ID/packages-microsoft-prod.deb
            sudo dpkg -i packages-microsoft-prod.deb
            rm packages-microsoft-prod.deb
            
            # Install .NET SDK
            sudo apt-get update
            sudo apt-get install -y apt-transport-https
            sudo apt-get update
            sudo apt-get install -y dotnet-sdk-7.0
            ;;
        fedora|rhel|centos)
            # Add Microsoft package repository
            sudo rpm -Uvh https://packages.microsoft.com/config/centos/7/packages-microsoft-prod.rpm
            
            # Install .NET SDK
            sudo yum install -y dotnet-sdk-7.0
            ;;
        opensuse*|sles)
            # Install .NET SDK from zypper
            sudo zypper install -y dotnet-sdk-7.0
            ;;
        arch|manjaro)
            # Install .NET SDK from pacman
            sudo pacman -Sy dotnet-sdk aspnet-runtime --noconfirm
            ;;
        *)
            echo -e "${YELLOW}Unsupported distribution for automatic .NET installation.${NC}"
            echo -e "${YELLOW}Please install .NET SDK 7.0 manually from https://dotnet.microsoft.com/download${NC}"
            read -p "Press Enter to continue once .NET SDK is installed..." 
            ;;
    esac
    
    # Verify .NET installation
    dotnet --version
    if [ $? -ne 0 ]; then
        echo -e "${RED}.NET SDK installation failed. Please install .NET SDK 7.0 manually.${NC}"
        exit 1
    fi
}

# Install dependencies
install_dependencies() {
    echo -e "${GREEN}Installing dependencies...${NC}"
    
    case $DISTRO in
        ubuntu|debian|linuxmint)
            sudo apt-get update
            sudo apt-get install -y unzip p7zip-full unrar-free libssl-dev zlib1g-dev libicu-dev build-essential
            ;;
        fedora|rhel|centos)
            sudo yum install -y unzip p7zip unrar openssl-devel zlib-devel libicu-devel gcc-c++ make
            ;;
        opensuse*|sles)
            sudo zypper install -y unzip p7zip unrar libopenssl-devel zlib-devel libicu-devel gcc-c++ make
            ;;
        arch|manjaro)
            sudo pacman -Sy unzip p7zip unrar openssl zlib icu gcc make --noconfirm
            ;;
        *)
            echo -e "${YELLOW}Unsupported distribution for automatic dependency installation.${NC}"
            echo -e "${YELLOW}Please make sure the following are installed: unzip, p7zip, unrar, ssl, zlib, icu, gcc/g++ and make.${NC}"
            ;;
    esac
    
    # Install Entity Framework Core CLI tools if needed (version 7.x)
    if ! command -v dotnet-ef &> /dev/null; then
        echo -e "${GREEN}Installing Entity Framework Core tools compatible with .NET 7...${NC}"
        dotnet tool install --global dotnet-ef --version 7.0.15
    fi
}

# Check if .NET is already installed
if command -v dotnet >/dev/null 2>&1; then
    dotnet_version=$(dotnet --version)
    echo -e "${GREEN}.NET SDK version $dotnet_version is already installed.${NC}"
else
    install_dotnet
fi

# Install dependencies
install_dependencies

# Create installation directory
INSTALL_DIR="$HOME/.local/share/transmission-rss-manager"
mkdir -p "$INSTALL_DIR"

# Clone or download the application
echo -e "${GREEN}Downloading TransmissionRssManager...${NC}"
if [ -d "/opt/develop/transmission-rss-manager/TransmissionRssManager" ]; then
    # We're running from the development directory
    cp -r /opt/develop/transmission-rss-manager/TransmissionRssManager/* "$INSTALL_DIR/"
else
    # Download and extract release
    wget -O transmission-rss-manager.zip https://github.com/yourusername/transmission-rss-manager/releases/latest/download/transmission-rss-manager.zip
    unzip transmission-rss-manager.zip -d "$INSTALL_DIR"
    rm transmission-rss-manager.zip
fi

# Install required NuGet packages (with versions compatible with .NET 7)
echo -e "${GREEN}Installing required NuGet packages...${NC}"
cd "$INSTALL_DIR"
dotnet add package Microsoft.AspNetCore.OpenApi --version 7.0.13
dotnet add package Swashbuckle.AspNetCore --version 6.5.0
dotnet add package System.ServiceModel.Syndication --version 7.0.0

# Build the application
echo -e "${GREEN}Building TransmissionRssManager...${NC}"
dotnet build -c Release

# Create configuration directory
CONFIG_DIR="$HOME/.config/transmission-rss-manager"
mkdir -p "$CONFIG_DIR"

# Create desktop entry
DESKTOP_FILE="$HOME/.local/share/applications/transmission-rss-manager.desktop"
echo "[Desktop Entry]
Name=Transmission RSS Manager
Comment=RSS Feed Manager for Transmission BitTorrent Client
Exec=dotnet $INSTALL_DIR/bin/Release/net7.0/TransmissionRssManager.dll
Icon=transmission
Terminal=false
Type=Application
Categories=Network;P2P;" > "$DESKTOP_FILE"

# Create systemd service for user
SERVICE_DIR="$HOME/.config/systemd/user"
mkdir -p "$SERVICE_DIR"

echo "[Unit]
Description=Transmission RSS Manager
After=network.target

[Service]
ExecStart=dotnet $INSTALL_DIR/bin/Release/net7.0/TransmissionRssManager.dll
Restart=on-failure
RestartSec=10
SyslogIdentifier=transmission-rss-manager

[Install]
WantedBy=default.target" > "$SERVICE_DIR/transmission-rss-manager.service"

# Reload systemd
systemctl --user daemon-reload

# Create launcher script
LAUNCHER="$HOME/.local/bin/transmission-rss-manager"
mkdir -p "$HOME/.local/bin"

echo "#!/bin/bash
dotnet $INSTALL_DIR/bin/Release/net7.0/TransmissionRssManager.dll" > "$LAUNCHER"
chmod +x "$LAUNCHER"

echo -e "${GREEN}Installation completed!${NC}"
echo -e "${GREEN}You can run TransmissionRssManager in these ways:${NC}"
echo -e "  * Command: ${YELLOW}transmission-rss-manager${NC}"
echo -e "  * Service: ${YELLOW}systemctl --user start transmission-rss-manager${NC}"
echo -e "  * Enable service on startup: ${YELLOW}systemctl --user enable transmission-rss-manager${NC}"
echo -e "  * Web interface will be available at: ${YELLOW}http://localhost:5000${NC}"

# Start the application
read -p "Do you want to start the application now? (y/n) " -n 1 -r
echo
if [[ $REPLY =~ ^[Yy]$ ]]; then
    systemctl --user start transmission-rss-manager
    echo -e "${GREEN}TransmissionRssManager service started.${NC}"
    echo -e "${GREEN}Open http://localhost:5000 in your browser.${NC}"
else
    echo -e "${YELLOW}You can start the application later using: systemctl --user start transmission-rss-manager${NC}"
fi