using System;
using System.IO;
using System.Text.Json;
using System.Threading.Tasks;
using Microsoft.Extensions.Logging;
using TransmissionRssManager.Core;

namespace TransmissionRssManager.Services
{
    public class ConfigService : IConfigService
    {
        private readonly ILogger<ConfigService> _logger;
        private readonly string _configPath;
        private AppConfig _cachedConfig;

        public ConfigService(ILogger<ConfigService> logger)
        {
            _logger = logger;
            
            // Get config directory
            string homeDir = Environment.GetFolderPath(Environment.SpecialFolder.UserProfile);
            string configDir = Path.Combine(homeDir, ".config", "transmission-rss-manager");
            
            // Ensure directory exists
            if (!Directory.Exists(configDir))
            {
                Directory.CreateDirectory(configDir);
            }
            
            _configPath = Path.Combine(configDir, "config.json");
            _cachedConfig = LoadConfiguration();
        }

        public AppConfig GetConfiguration()
        {
            return _cachedConfig;
        }

        public async Task SaveConfigurationAsync(AppConfig config)
        {
            _cachedConfig = config;
            
            var options = new JsonSerializerOptions
            {
                WriteIndented = true
            };
            
            string json = JsonSerializer.Serialize(config, options);
            await File.WriteAllTextAsync(_configPath, json);
            
            _logger.LogInformation("Configuration saved successfully");
        }

        private AppConfig LoadConfiguration()
        {
            if (!File.Exists(_configPath))
            {
                _logger.LogInformation("No configuration file found, creating default");
                var defaultConfig = CreateDefaultConfig();
                SaveConfigurationAsync(defaultConfig).Wait();
                return defaultConfig;
            }

            try
            {
                string json = File.ReadAllText(_configPath);
                var config = JsonSerializer.Deserialize<AppConfig>(json);
                
                if (config == null)
                {
                    _logger.LogWarning("Failed to deserialize config, creating default");
                    return CreateDefaultConfig();
                }
                
                return config;
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Error loading configuration");
                return CreateDefaultConfig();
            }
        }

        private AppConfig CreateDefaultConfig()
        {
            string homeDir = Environment.GetFolderPath(Environment.SpecialFolder.UserProfile);
            
            return new AppConfig
            {
                Transmission = new TransmissionConfig
                {
                    Host = "localhost",
                    Port = 9091,
                    Username = "",
                    Password = ""
                },
                AutoDownloadEnabled = false,
                CheckIntervalMinutes = 30,
                DownloadDirectory = Path.Combine(homeDir, "Downloads"),
                MediaLibraryPath = Path.Combine(homeDir, "Media"),
                PostProcessing = new PostProcessingConfig
                {
                    Enabled = false,
                    ExtractArchives = true,
                    OrganizeMedia = true,
                    MinimumSeedRatio = 1,
                    MediaExtensions = new System.Collections.Generic.List<string> { ".mp4", ".mkv", ".avi" }
                }
            };
        }
    }
}