using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using Microsoft.Extensions.Logging;
using TransmissionRssManager.Core;

namespace TransmissionRssManager.Services
{
    /// <summary>
    /// Interface for the metrics service that provides dashboard statistics and performance data
    /// </summary>
    public interface IMetricsService
    {
        Task<Dictionary<string, object>> GetDashboardStatsAsync();
        Task<Dictionary<string, long>> EstimateDiskUsageAsync();
        Task<Dictionary<string, object>> GetSystemStatusAsync();
    }
    
    /// <summary>
    /// Service that provides metrics and statistics about downloads, system status, and performance
    /// </summary>
    public class MetricsService : IMetricsService
    {
        private readonly ILogger<MetricsService> _logger;
        private readonly ITransmissionClient _transmissionClient;
        private readonly IConfigService _configService;
        
        public MetricsService(
            ILogger<MetricsService> logger,
            ITransmissionClient transmissionClient,
            IConfigService configService)
        {
            _logger = logger;
            _transmissionClient = transmissionClient;
            _configService = configService;
        }
        
        /// <summary>
        /// Gets dashboard statistics including active downloads, upload/download speeds, etc.
        /// </summary>
        public async Task<Dictionary<string, object>> GetDashboardStatsAsync()
        {
            try
            {
                var stats = new Dictionary<string, object>();
                var torrents = await _transmissionClient.GetTorrentsAsync();
                
                // Calculate basic stats
                stats["TotalTorrents"] = torrents.Count;
                stats["ActiveDownloads"] = torrents.Count(t => t.Status == "Downloading");
                stats["SeedingTorrents"] = torrents.Count(t => t.Status == "Seeding");
                stats["CompletedTorrents"] = torrents.Count(t => t.IsFinished);
                stats["TotalDownloaded"] = torrents.Sum(t => t.DownloadedEver);
                stats["TotalUploaded"] = torrents.Sum(t => t.UploadedEver);
                stats["DownloadSpeed"] = torrents.Sum(t => t.DownloadSpeed);
                stats["UploadSpeed"] = torrents.Sum(t => t.UploadSpeed);
                
                // Calculate total size
                long totalSize = torrents.Sum(t => t.TotalSize);
                stats["TotalSize"] = totalSize;
                
                return stats;
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Error getting dashboard stats");
                return new Dictionary<string, object>
                {
                    ["Error"] = ex.Message,
                    ["TotalTorrents"] = 0,
                    ["ActiveDownloads"] = 0,
                    ["SeedingTorrents"] = 0,
                    ["CompletedTorrents"] = 0
                };
            }
        }
        
        /// <summary>
        /// Estimates disk usage for torrents and available space
        /// </summary>
        public async Task<Dictionary<string, long>> EstimateDiskUsageAsync()
        {
            try
            {
                // Get disk usage from torrents
                var torrents = await _transmissionClient.GetTorrentsAsync();
                long totalSize = torrents.Sum(t => t.TotalSize);
                
                // Calculate available space in download directory
                string downloadDir = _configService.GetConfiguration().DownloadDirectory;
                long availableSpace = 0;
                
                if (!string.IsNullOrEmpty(downloadDir) && System.IO.Directory.Exists(downloadDir))
                {
                    try
                    {
                        var driveInfo = new System.IO.DriveInfo(System.IO.Path.GetPathRoot(downloadDir));
                        availableSpace = driveInfo.AvailableFreeSpace;
                    }
                    catch (Exception ex)
                    {
                        _logger.LogWarning(ex, $"Error getting available disk space for {downloadDir}");
                    }
                }
                
                return new Dictionary<string, long>
                {
                    ["activeTorrentsSize"] = totalSize,
                    ["availableSpace"] = availableSpace
                };
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Error estimating disk usage");
                return new Dictionary<string, long>
                {
                    ["activeTorrentsSize"] = 0,
                    ["availableSpace"] = 0
                };
            }
        }
        
        /// <summary>
        /// Gets system status including Transmission connection state
        /// </summary>
        public async Task<Dictionary<string, object>> GetSystemStatusAsync()
        {
            var config = _configService.GetConfiguration();
            
            var status = new Dictionary<string, object>
            {
                ["TransmissionConnected"] = false,
                ["AutoDownloadEnabled"] = config.AutoDownloadEnabled,
                ["PostProcessingEnabled"] = config.PostProcessing.Enabled,
                ["CheckIntervalMinutes"] = config.CheckIntervalMinutes
            };
            
            try
            {
                // Try to connect to Transmission to check if it's available
                var torrents = await _transmissionClient.GetTorrentsAsync();
                
                status["TransmissionConnected"] = true;
                status["TransmissionTorrentCount"] = torrents.Count;
                
                // Count torrents by status
                status["ActiveTorrentCount"] = torrents.Count(t => t.Status == "Downloading");
                status["CompletedTorrentCount"] = torrents.Count(t => t.IsFinished);
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Error getting system status");
                status["TransmissionConnected"] = false;
                status["LastErrorMessage"] = ex.Message;
            }
            
            return status;
        }
    }
}