using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Net.Http;
using System.ServiceModel.Syndication;
using System.Text.Json;
using System.Text.RegularExpressions;
using System.Threading;
using System.Threading.Tasks;
using System.Xml;
using Microsoft.Extensions.Logging;
using Microsoft.Extensions.Hosting;
using TransmissionRssManager.Core;

namespace TransmissionRssManager.Services
{
    public class RssFeedManager : IRssFeedManager
    {
        private readonly ILogger<RssFeedManager> _logger;
        private readonly IConfigService _configService;
        private readonly ITransmissionClient _transmissionClient;
        private readonly HttpClient _httpClient;
        private readonly string _dataPath;
        private List<RssFeedItem> _items = new List<RssFeedItem>();

        public RssFeedManager(
            ILogger<RssFeedManager> logger,
            IConfigService configService,
            ITransmissionClient transmissionClient)
        {
            _logger = logger;
            _configService = configService;
            _transmissionClient = transmissionClient;
            _httpClient = new HttpClient();
            
            // Create data directory
            string homeDir = Environment.GetFolderPath(Environment.SpecialFolder.UserProfile);
            string dataDir = Path.Combine(homeDir, ".local", "share", "transmission-rss-manager");
            
            if (!Directory.Exists(dataDir))
            {
                Directory.CreateDirectory(dataDir);
            }
            
            _dataPath = Path.Combine(dataDir, "rss-items.json");
            LoadItems();
        }

        public Task<List<RssFeedItem>> GetAllItemsAsync()
        {
            return Task.FromResult(_items.OrderByDescending(i => i.PublishDate).ToList());
        }

        public Task<List<RssFeedItem>> GetMatchedItemsAsync()
        {
            return Task.FromResult(_items.Where(i => i.IsMatched).OrderByDescending(i => i.PublishDate).ToList());
        }

        public Task<List<RssFeed>> GetFeedsAsync()
        {
            var config = _configService.GetConfiguration();
            return Task.FromResult(config.Feeds);
        }

        public async Task AddFeedAsync(RssFeed feed)
        {
            feed.Id = Guid.NewGuid().ToString();
            feed.LastChecked = DateTime.MinValue;

            var config = _configService.GetConfiguration();
            config.Feeds.Add(feed);
            await _configService.SaveConfigurationAsync(config);
            
            // Initial fetch of feed items
            await FetchFeedAsync(feed);
        }

        public async Task RemoveFeedAsync(string feedId)
        {
            var config = _configService.GetConfiguration();
            var feed = config.Feeds.FirstOrDefault(f => f.Id == feedId);
            
            if (feed != null)
            {
                config.Feeds.Remove(feed);
                await _configService.SaveConfigurationAsync(config);
                
                // Remove items from this feed
                _items.RemoveAll(i => i.Id.StartsWith(feedId));
                await SaveItemsAsync();
            }
        }

        public async Task UpdateFeedAsync(RssFeed feed)
        {
            var config = _configService.GetConfiguration();
            var index = config.Feeds.FindIndex(f => f.Id == feed.Id);
            
            if (index != -1)
            {
                config.Feeds[index] = feed;
                await _configService.SaveConfigurationAsync(config);
            }
        }

        public async Task RefreshFeedsAsync(CancellationToken cancellationToken)
        {
            _logger.LogInformation("Starting RSS feed refresh");
            var config = _configService.GetConfiguration();
            
            foreach (var feed in config.Feeds)
            {
                if (cancellationToken.IsCancellationRequested)
                {
                    _logger.LogInformation("RSS refresh cancelled");
                    return;
                }
                
                try
                {
                    await FetchFeedAsync(feed);
                    
                    // Update last checked time
                    feed.LastChecked = DateTime.Now;
                    await _configService.SaveConfigurationAsync(config);
                }
                catch (Exception ex)
                {
                    _logger.LogError(ex, $"Error refreshing feed: {feed.Name}");
                }
            }
            
            // Check for matches and auto-download if enabled
            await ProcessMatchesAsync();
        }

        public async Task MarkItemAsDownloadedAsync(string itemId)
        {
            var item = _items.FirstOrDefault(i => i.Id == itemId);
            
            if (item != null)
            {
                item.IsDownloaded = true;
                await SaveItemsAsync();
            }
        }

        private async Task FetchFeedAsync(RssFeed feed)
        {
            _logger.LogInformation($"Fetching feed: {feed.Name}");
            
            try
            {
                var response = await _httpClient.GetStringAsync(feed.Url);
                using var stringReader = new StringReader(response);
                using var xmlReader = XmlReader.Create(stringReader);
                
                var syndicationFeed = SyndicationFeed.Load(xmlReader);
                
                foreach (var item in syndicationFeed.Items)
                {
                    var link = item.Links.FirstOrDefault()?.Uri.ToString() ?? "";
                    var torrentUrl = ExtractTorrentUrl(link, item.Title.Text);
                    
                    // Create a unique ID for this item
                    var itemId = $"{feed.Id}:{item.Id ?? Guid.NewGuid().ToString()}";
                    
                    // Check if we already have this item
                    if (_items.Any(i => i.Id == itemId))
                    {
                        continue;
                    }
                    
                    var feedItem = new RssFeedItem
                    {
                        Id = itemId,
                        Title = item.Title.Text,
                        Link = link,
                        Description = item.Summary?.Text ?? "",
                        PublishDate = item.PublishDate.DateTime,
                        TorrentUrl = torrentUrl,
                        IsDownloaded = false
                    };
                    
                    // Check if this item matches any rules
                    CheckForMatches(feedItem, feed.Rules);
                    
                    _items.Add(feedItem);
                }
                
                await SaveItemsAsync();
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, $"Error fetching feed: {feed.Name}");
                throw;
            }
        }

        private string ExtractTorrentUrl(string link, string title)
        {
            // Try to find a .torrent link
            if (link.EndsWith(".torrent", StringComparison.OrdinalIgnoreCase))
            {
                return link;
            }
            
            // If it's a magnet link, return it
            if (link.StartsWith("magnet:", StringComparison.OrdinalIgnoreCase))
            {
                return link;
            }
            
            // Return the link as is, we'll try to find the torrent on the page
            return link;
        }

        private void CheckForMatches(RssFeedItem item, List<string> rules)
        {
            foreach (var rule in rules)
            {
                try
                {
                    if (Regex.IsMatch(item.Title, rule, RegexOptions.IgnoreCase))
                    {
                        item.IsMatched = true;
                        item.MatchedRule = rule;
                        break;
                    }
                }
                catch (Exception ex)
                {
                    _logger.LogError(ex, $"Invalid regex rule: {rule}");
                }
            }
        }

        private async Task ProcessMatchesAsync()
        {
            var config = _configService.GetConfiguration();
            
            if (!config.AutoDownloadEnabled)
            {
                return;
            }
            
            var matchedItems = _items.Where(i => i.IsMatched && !i.IsDownloaded).ToList();
            
            foreach (var item in matchedItems)
            {
                try
                {
                    _logger.LogInformation($"Auto-downloading: {item.Title}");
                    
                    var torrentId = await _transmissionClient.AddTorrentAsync(
                        item.TorrentUrl, 
                        config.DownloadDirectory);
                    
                    item.IsDownloaded = true;
                }
                catch (Exception ex)
                {
                    _logger.LogError(ex, $"Error downloading torrent: {item.Title}");
                }
            }
            
            await SaveItemsAsync();
        }

        private void LoadItems()
        {
            if (!File.Exists(_dataPath))
            {
                _items = new List<RssFeedItem>();
                return;
            }

            try
            {
                var json = File.ReadAllText(_dataPath);
                var items = JsonSerializer.Deserialize<List<RssFeedItem>>(json);
                _items = items ?? new List<RssFeedItem>();
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Error loading RSS items");
                _items = new List<RssFeedItem>();
            }
        }

        private async Task SaveItemsAsync()
        {
            try
            {
                var options = new JsonSerializerOptions
                {
                    WriteIndented = true
                };
                
                var json = JsonSerializer.Serialize(_items, options);
                await File.WriteAllTextAsync(_dataPath, json);
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Error saving RSS items");
            }
        }
    }

    public class RssFeedBackgroundService : BackgroundService
    {
        private readonly ILogger<RssFeedBackgroundService> _logger;
        private readonly IRssFeedManager _rssFeedManager;
        private readonly IConfigService _configService;

        public RssFeedBackgroundService(
            ILogger<RssFeedBackgroundService> logger,
            IRssFeedManager rssFeedManager,
            IConfigService configService)
        {
            _logger = logger;
            _rssFeedManager = rssFeedManager;
            _configService = configService;
        }

        protected override async Task ExecuteAsync(CancellationToken stoppingToken)
        {
            _logger.LogInformation("RSS feed background service started");
            
            while (!stoppingToken.IsCancellationRequested)
            {
                try
                {
                    await _rssFeedManager.RefreshFeedsAsync(stoppingToken);
                }
                catch (Exception ex)
                {
                    _logger.LogError(ex, "Error refreshing RSS feeds");
                }
                
                var config = _configService.GetConfiguration();
                var interval = TimeSpan.FromMinutes(config.CheckIntervalMinutes);
                
                _logger.LogInformation($"Next refresh in {interval.TotalMinutes} minutes");
                await Task.Delay(interval, stoppingToken);
            }
        }
    }
}